@@ -15,6 +15,7 @@
 #' @param hidden.layer vector for specifying hidden layers and number of neurons.
 #' @param ID string for identifier variable
 #' @param stepmax maximum number of steps.
+#' @param act.fct "logistic" or "tanh activation function.
 #'
 #' @return trained complier model object
 #' @export
@@ -24,6 +25,7 @@ neuralnet_complier_mod<-function(complier.formula,
                                  treat.var,
                                  algorithm = "rprop+",
                                  hidden.layer = c(4,2),
+                                 act.fct = "logistic",
                                  ID = NULL,
                                  stepmax = 1e+08){
   if (!is.null(ID)){
@@ -40,6 +42,7 @@ neuralnet_complier_mod<-function(complier.formula,
                                                  algorithm = algorithm,
                                                  hidden = hidden.layer,
                                                  linear.output = FALSE,
+                                                 act.fct = act.fct,
                                                  stepmax = stepmax)
   return(neuralnet.complier.mod)
 }
@@ -92,6 +95,9 @@ neuralnet_predict<-function(neuralnet.complier.mod,
 #' @param algorithm neural network algorithm, default set to `"rprop+"`.
 #' @param hidden.layer vector specifying hidden layers and number of neurons.
 #' @param stepmax maximum number of steps for training model.
+#' @param act.fct "logistic" or "tanh activation function.
+#' @param err.fct "sse" for sum of squared errors or "ce" for cross-entropy.
+#' @param linear.output logical for whether output (outcome variable) is linear or not.
 #'
 #' @return trained response model object
 #' @export
@@ -102,6 +108,9 @@ neuralnet_response_model <- function(response.formula,
                                      compl.var,
                                      algorithm = "rprop+",
                                      hidden.layer = c(4,2),
+                                     act.fct = "logistic",
+                                     err.fct = "sse",
+                                     linear.output = TRUE,
                                      stepmax = 1e+08){
 
   variables <- all.vars(response.formula)
@@ -118,7 +127,9 @@ neuralnet_response_model <- function(response.formula,
                                             data = exp.compliers,
                                             hidden = hidden.layer,
                                             algorithm = algorithm,
-                                            linear.output = FALSE,
+                                            linear.output = linear.output,
+                                            act.fct = act.fct,
+                                            err.fct = err.fct,
                                             stepmax = stepmax)
   return(neuralnet.response.mod)
 }
@@ -136,7 +147,7 @@ neuralnet_response_model <- function(response.formula,
 #' \code{neuralnet_response_model}.
 #' @param ID string for identifier variable.
 #' @param cluster string for clustering variable (currently unused).
-#' @param binary.outcome logical specifying predicted outcome variable will take
+#' @param binary.preds logical specifying predicted outcome variable will take
 #' binary values or proportions.
 #'
 #' @return `data.frame` of predicted outcomes of response variable from
@@ -147,7 +158,7 @@ neuralnet_pattc_counterfactuals <- function (pop.data,
                                              neuralnet.response.mod,
                                              ID = NULL,
                                              cluster = NULL,
-                                             binary.outcome = FALSE){
+                                             binary.preds = FALSE){
 
   compl.var <- pop.data$compl_var
   covariates <- all.vars(pop.data$response_formula)[-1]
@@ -155,18 +166,20 @@ neuralnet_pattc_counterfactuals <- function (pop.data,
   popdata$c <- popdata[,compl.var]
   popdata_comp <- popdata[which(popdata$c==1),]
 
-  pop.tr.counterfactual <- cbind(rep(1, nrow(popdata_comp)), popdata_comp[, covariates])
+  pop.tr.counterfactual <- cbind(rep(1, nrow(popdata_comp)), 
+                                 popdata_comp[, covariates])
   colnames(pop.tr.counterfactual) <- c(compl.var, covariates)
-  pop.ctrl.counterfactual <- cbind(rep(0, nrow(popdata_comp)), popdata_comp[, covariates])
+  pop.ctrl.counterfactual <- cbind(rep(0, nrow(popdata_comp)), 
+                                   popdata_comp[, covariates])
   colnames(pop.ctrl.counterfactual) <- c(compl.var, covariates)
 
   Y.hat.0 <- predict(neuralnet.response.mod, pop.ctrl.counterfactual)
   Y.hat.1 <- predict(neuralnet.response.mod, pop.tr.counterfactual)
 
-  if (binary.outcome) {
+  if (binary.preds) {
     neuralnetpredict.max.0 <- max.col(Y.hat.0) - 1
     neuralnetpredict.max.1 <- max.col(Y.hat.1) - 1
-  } else if (!binary.outcome) {
+  } else if (!binary.preds) {
     neuralnetpredict.max.0 <- Y.hat.0[,2]
     neuralnetpredict.max.1 <- Y.hat.1[,2]
   }
@@ -201,10 +214,10 @@ neuralnet_pattc_counterfactuals <- function (pop.data,
 #' compliance variable
 #' @param treat.var string for treatment variable.
 #' @param compl.var string for compliance variable
-#' @param compl.algorithm string for algorithim to train neural network for
+#' @param compl.algorithm string for algorithm to train neural network for
 #' compliance model. Default set to `"rprop+"`. See (`neuralnet` package for
 #' available algorithms).
-#' @param response.algorithm string for algorithim to train neural network for
+#' @param response.algorithm string for algorithm to train neural network for
 #' response model. Default set to `"rprop+"`. See (`neuralnet` package for
 #' available algorithms).
 #' @param compl.hidden.layer vector for specifying hidden layers and number of
@@ -215,22 +228,26 @@ neuralnet_pattc_counterfactuals <- function (pop.data,
 #' @param response.stepmax maximum number of steps for response model
 #' @param ID string for identifier variable
 #' @param cluster string for cluster variable.
-#' @param binary.outcome logical specifying predicted outcome variable will take
+#' @param binary.preds logical specifying predicted outcome variable will take
 #' binary values or proportions.
 #' @param bootstrap logical for bootstrapped PATT-C.
 #' @param nboot number of bootstrapped samples
+#' @param compl.act.fct "logistic" or "tanh" activation function for complier model.
+#' @param response.err.fct "sse" for sum of squared errors or "ce" for cross-entropy for response model.
+#' @param response.act.fct "logistic" or "tanh" activation function for response model.
+#' @param linear.output logical for whether output (outcome variable) is linear or not for response model.
 #'
-#' @return `pattc_deepneural` class object of results of t test as PATTC estimate.
+#' @return `pattc_neural` class object of results of t test as PATTC estimate.
 #' @export
 #'
 #' @examples
-#' \donttest{
 #' # load datasets
 #' data(exp_data) #experimental data
 #' data(pop_data) #population data
 #' # specify models and estimate PATTC
 #' set.seed(123456)
-#' pattc_neural <- pattc_deepneural(response.formula = support_war ~ age + female +
+#'
+#' pattc_neural_boot <- pattc_neural(response.formula = support_war ~ age + female +
 #'                                income + education +  employed + married +
 #'                                hindu + job_loss,
 #'                                exp.data = exp_data,
@@ -239,57 +256,42 @@ neuralnet_pattc_counterfactuals <- function (pop.data,
 #'                                compl.var = "compliance",
 #'                                compl.algorithm = "rprop+",
 #'                                response.algorithm = "rprop+",
-#'                                compl.hidden.layer = c(4,2),
-#'                                response.hidden.layer = c(4,2),
+#'                                compl.hidden.layer = c(2),
+#'                                response.hidden.layer = c(2),
 #'                                compl.stepmax = 1e+09,
 #'                                response.stepmax = 1e+09,
 #'                                ID = NULL,
 #'                                cluster = NULL,
-#'                                binary.outcome = FALSE)
-#'
-#' print(pattc_neural)
-#'
-#' pattc_neural_boot <- pattc_deepneural(response.formula = support_war ~ age + female +
-#'                                income + education +  employed + married +
-#'                                hindu + job_loss,
-#'                                exp.data = exp_data,
-#'                                pop.data = pop_data,
-#'                                treat.var = "strong_leader",
-#'                                compl.var = "compliance",
-#'                                compl.algorithm = "rprop+",
-#'                                response.algorithm = "rprop+",
-#'                                compl.hidden.layer = c(4,2),
-#'                                response.hidden.layer = c(4,2),
-#'                                compl.stepmax = 1e+09,
-#'                                response.stepmax = 1e+09,
-#'                                ID = NULL,
-#'                                cluster = NULL,
-#'                                binary.outcome = FALSE,
+#'                                binary.preds = FALSE,
 #'                                bootstrap = TRUE,
-#'                                nboot = 2000)
+#'                                nboot = 1000)
 #'
-#' print(pattc_neural_boot)
-#'
-#' }
-#'
-pattc_deepneural <- function(response.formula,
-                         exp.data,
-                         pop.data,
-                         treat.var,
-                         compl.var,
-                         compl.algorithm = "rprop+",
-                         response.algorithm = "rprop+",
-                         compl.hidden.layer = c(4,2),
-                         response.hidden.layer = c(4,2),
-                         compl.stepmax = 1e+08,
-                         response.stepmax = 1e+08,
-                         ID = NULL,
-                         cluster = NULL,
-                         binary.outcome = FALSE,
-                         bootstrap = FALSE,
-                         nboot = 1000)
+pattc_neural <- function(response.formula,
+                             exp.data,
+                             pop.data,
+                             treat.var,
+                             compl.var,
+                             compl.algorithm = "rprop+",
+                             response.algorithm = "rprop+",
+                             compl.hidden.layer = c(4,2),
+                             response.hidden.layer = c(4,2),
+                             compl.act.fct = "logistic",
+                             response.err.fct = "sse",
+                             response.act.fct = "logistic",
+                             linear.output = TRUE,
+                             compl.stepmax = 1e+08,
+                             response.stepmax = 1e+08,
+                             ID = NULL,
+                             cluster = NULL,
+                             binary.preds = FALSE,
+                             bootstrap = FALSE,
+                             nboot = 1000)
 
 {
+  if (response.err.fct == "ce" & linear.output == TRUE){
+    stop("cross-entropy error function cannot be used with linear output.
+         Please set linear.output = FALSE")
+  }
   expdata <- expcall(response.formula,
                      treat.var = treat.var,
                      compl.var = compl.var,
@@ -310,6 +312,10 @@ pattc_deepneural <- function(response.formula,
   neuralnet.compl.mod <- neuralnet_complier_mod(complier.formula = compl.formula,
                                                 exp.data = expdata$exp_data,
                                                 treat.var = treat.var,
+                                                algorithm = compl.algorithm,
+                                                hidden.layer = compl.hidden.layer,
+                                                act.fct = compl.act.fct,
+                                                ID = ID,
                                                 stepmax = compl.stepmax)
 
   compliers <- neuralnet_predict(neuralnet.complier.mod = neuralnet.compl.mod,
@@ -321,16 +327,22 @@ pattc_deepneural <- function(response.formula,
                                                   compl.var = compl.var,
                                                   exp.data = expdata$exp_data,
                                                   neuralnet.compliers = compliers,
+                                                  algorithm = response.algorithm,
+                                                  hidden.layer = response.hidden.layer,
+                                                  act.fct = response.act.fct,
+                                                  err.fct = response.err.fct,
+                                                  linear.output = linear.output,
                                                   stepmax = response.stepmax)
+  
   message("Predicting response and estimating PATT-C")
   counterfactuals <- neuralnet_pattc_counterfactuals(popdata,
                                                      neural.response.mod,
-                                                     binary.outcome = binary.outcome)
+                                                     binary.preds = binary.preds)
 
   outcome.var <- all.vars(response.formula)[1]
   dummy <- length(levels(as.factor(expdata$exp_data[,outcome.var])) )
 
-  if (binary.outcome) {
+  if (binary.preds) {
     Y_hat1_0s <- sum(counterfactuals$Y_hat0)
     nY_hat0 <- length(counterfactuals$Y_hat0)
     Y_hat1_1s <- sum(counterfactuals$Y_hat1)
@@ -349,7 +361,7 @@ pattc_deepneural <- function(response.formula,
                  pattc_xsq$method,
                  statistic)
   }
-  else if (!binary.outcome){
+  else if (!binary.preds){
     if (bootstrap) {
       bootResults <- matrix(NA, nrow = nboot, ncol = ncol(counterfactuals)+1)
       for (i in seq_len(nboot)){
@@ -399,22 +411,22 @@ pattc_deepneural <- function(response.formula,
                   "pop_counterfactual" = counterfactuals,
                   "PATT_C" = pattc)
 
-  class(model.out)<-"pattc_deepneural"
+  class(model.out)<-"pattc_neural"
   return(model.out)
 }
 
-#' print.pattc_deepneural
+#' print.pattc_neural
 #'
 #' @description
-#' Print method for \code{pattc_deepneural}
-#' @param x `pattc_deepneural` class object from \code{pattc_deepneural}
+#' Print method for \code{pattc_neural}
+#' @param x `pattc_neural` class object from \code{pattc_neural}
 #' @param ... additional parameter
 #'
 #' @return list of model results
 #' @export
 #'
 
-print.pattc_deepneural <- function(x, ...){
+print.pattc_neural <- function(x, ...){
   cat("Method:\n")
   cat("Deep Neural PATT-C\n")
   cat("Formula:\n")
